import os
import datetime
import xml
from xml.parsers.expat import *
from xml.dom.minidom import parse, parseString
from dynastie.generators.generator import DynastieGenerator, StrictUTF8Writer
from django.db import models

class Index(DynastieGenerator):

    cur_page = 0
    nb_pages = 0
    cur_post = 0
    posts_per_page = 0
    filename = 'index'
    dirname = ''
    blog = None

    def createReplace(self, posts, dom, root, replace_elem):
        if not replace_elem.hasAttribute('div_name'):
            self.addError('No attribute div_name for a replace tag')
            return

        div_element = replace_elem.cloneNode(True)
        div_element.tagName = replace_elem.getAttribute('div_name')
        div_element.removeAttribute('div_name')
        for key,value in replace_elem.attributes.items():
            if key == 'div_name': continue

            value = value.replace('dyn:blog_id', str(self.blog.id))
                
            div_element.setAttribute(key, value)

        root.replaceChild(div_element, replace_elem)
        return div_element

    def createNavigation(self, posts, dom, root, node):
        if self.nb_pages == 0 or self.nb_pages == 1:
            return None
        
        if self.dirname != '':
            if self.dirname.startswith('/'):
                href = '<a href="' + self.dirname + '/' + self.filename
            else:
                href = '<a href="/' + self.dirname + '/' + self.filename
        else:
            href = '<a href="/' + self.filename

        nav = ''
        if self.cur_page != 0:
            nav = nav + href + '.html">&lt;&lt; First</a>  '
            if self.cur_page == 1:
                nav = nav + href + '.html">&lt; Prev</a> '
            else:
                nav = nav + href + str(self.cur_page-1) + '.html">&lt; Prev</a> '

        start = self.cur_page-5
        if start < 0:
            start = 0
        end = start + 10
        
        if end > self.nb_pages:
            end = self.nb_pages

        for i in range(start, end):
            if i == self.cur_page:
                nav = nav + str(i+1) + ' '
            else:
                if i == 0:
                    nav = nav + href + '.html">1</a> '
                else:
                    nav = nav + href + str(i) + '.html">' + str(i+1) + '</a> '

        if self.cur_page < self.nb_pages-1:
            nav = nav + href + str(self.cur_page+1) + '.html">Next &gt;</a>  '
            nav = nav + href + str(self.nb_pages-1) + '.html">Last &gt;&gt;</a>'

        new_dom = parseString('<div class="navigation">' + nav + '</div>')
        new_node = new_dom.getElementsByTagName('div')[0]
        res = new_node.cloneNode(True)
        root.replaceChild(res, node)
        
        return res

    def createPost(self, post, dom, post_elem, root):
        values = {}
        values['title'] = self.createLinkElem(dom, post.getPath(), post.title)
        values['author'] = post.author.first_name + ' ' + post.author.last_name
        values['date'] = post.creation_date.strftime('%A, %d %B %Y %H:%m')
        values['post_content'] = ''

        blog = post.blog
        blog.create_paths()

        filename = blog.src_path + '/_post/' + str(post.id)

        if not os.path.exists(filename):
            self.addError('File does not exists ' + filename)
            return

        f = open(filename, 'rb')
        post_content = f.read()
        f.close()

        while True:
            start = post_content.find('<dyn:code')
            end = post_content.find('</dyn:code>')

            if start == -1: break

            if end < start:
                self.addError('Invalid <dyn:code> tags in ' + filename)
                break
            
            try:
                dom2 = parseString(post_content[start:end+11])
            except xml.dom.DOMException as e:
                self.addError('Error parsing ' + filename)
                break
            
            res = self.createCode(dom2, dom2.firstChild)
            post_content = post_content.replace(post_content[start:end+11], res)

        self.simpleTransform(values, dom, post_elem, root)
        
        content_nodes = post_elem.getElementsByTagName('div')
        post_transform = ['post_content']
        for content_node in content_nodes:
            the_class = content_node.getAttribute('class')
            if not the_class in post_transform:
                continue
            new_node = dom.createTextNode(post_content)
            content_node.appendChild(new_node)
        
    def createPosts(self, posts, dom, root, node):
        posts_elem = self.createElement(dom, 'posts')
        for i in range(0, self.posts_per_page):
            post_elem = self.createElement(dom, 'post')
            if len(posts) > self.cur_post:
                self.createPost(posts[self.cur_post], dom, post_elem, node)
            else:
                post_elem = self.createElement(dom, '', '<b>No posts yet</b>')
            posts_elem.appendChild(post_elem)

            # Parse inner HTML
            self._parse(self.hooks, posts, dom, post_elem)

            self.cur_post = self.cur_post + 1
            if self.cur_post == len(posts):
                break
        root.replaceChild(posts_elem, node)
        return posts_elem

    def createRecents(self, posts, dom, root, node):
        if self.cur_post == len(posts):
            root.removeChild(node)
            return

        if node.hasAttribute("limit"):
            nb_recents = int(node.getAttribute("limit"))
        else:
            nb_recents = 5
        recents_elem = self.createElement(dom, 'recents')
        for child in node.childNodes:
            recents_elem.appendChild(child.cloneNode(True))
        list_elem = dom.createElement('ul')
        for i in range(0, nb_recents):
            post_elem = dom.createElement('li')
            if self.cur_post+i < len(posts):
                post = posts[self.cur_post+i]
                link_elem = self.createLinkElem(dom, post.getPath(), post.title)
                post_elem.appendChild(link_elem)
            else:
                break
            list_elem.appendChild(post_elem)

        recents_elem.appendChild(list_elem)
        root.replaceChild(recents_elem, node)
        
        return recents_elem

    def createTags(self, posts, dom, root, node):
        from dynastie.models import Post
        tags_elem = self.createElement(dom, 'tags')
        create_link = (node.getAttribute('link') == '1')
        if type(posts) == models.query.QuerySet:
            if len(posts) > self.cur_post:
                cur_post = posts[self.cur_post]
            else:
                cur_post = None
        elif type(posts) == Post:
            cur_post = posts
        else:
            cur_post = None

        if not cur_post is None:
            for tag in cur_post.tags.all():
                if create_link:
                    tag_elem = self.createElement(dom, 'tag')
                    link_elem = self.createLinkElem(dom, '/tag/' + tag.name_slug, '#' + tag.name)
                    tag_elem.appendChild(link_elem)
                else:
                    tag_elem = self.createElement(dom, 'tag', '#' + tag.name)
                tags_elem.appendChild(tag_elem)

            if len(cur_post.tags.all()) == 0:
                root.removeChild(node)
                return None
            else:
                root.replaceChild(tags_elem, node)
        else:
            root.removeChild(node)
            return None

        return tags_elem

    def createCode(self, dom, node):
        try:
            from pygments import highlight
            from pygments.util import ClassNotFound
            from pygments.lexers import get_lexer_by_name
            from pygments.formatters import get_formatter_by_name
        except ImportError:
            self.addError('Pygments package is missing, please install it in order to use <dyn:code>')
            return

        language = node.getAttribute('language')
        
        if language is None:
            self.addWarning('No language defined for <dyn:code> assuming C language')
            language = "c"
        else:
            language = language.lower()

        lexer_options = {}
        try:
            lexer = get_lexer_by_name(language, **lexer_options)
        except ClassNotFound:
            self.addWarning('Language ' + language + ' not supported by current version of pygments')
            lexer = get_lexer_by_name('c', **lexer_options)
        
        formatter_options = {'classprefix' : 'color_emacs_', 'style' : 'emacs'}

        for k in node.attributes.keys():
            attr = node.attributes[k]
            if attr.prefix != '': continue
            if attr.name == 'language': continue
            name = attr.name
            value = attr.value
            if name == 'colouring': 
                name = 'style'
                formatter_options['classprefix'] = 'color_' + value + '_' 
            formatter_options[name] = value

        formatter = get_formatter_by_name('html', **formatter_options)

        lexer.encoding = 'utf-8'
        formatter.encoding = 'utf-8'
        
        writer = StrictUTF8Writer()
        node.firstChild.writexml(writer)
        code = writer.getvalue().encode('utf-8')

        r,w = os.pipe()
        r,w=os.fdopen(r,'r',0), os.fdopen(w,'w',0)
        highlight(code, lexer, formatter, w)
        w.close()
        
        code = r.read()
        r.close()

        # Remove <pre> after <div class="highlight">
        code = code[28:-13]
        code = '<div class="highlight">' + code + '</div>'

        return code

    def generate(self, blog, src, output):
        from dynastie.models import Post, Blog

        self.hooks = {'posts' : self.createPosts,
                      'navigation' : self.createNavigation,
                      'recents' : self.createRecents,
                      'tags' : self.createTags,
                      'replace' : self.createReplace}

        self.blog = blog

        if not os.path.exists(src + '/_index.html'):
            self.addError('No _index.html found, exiting')
            return self.report

        try:
            dom = parse(src + '/_index.html')
        except xml.dom.DOMException as e:
            self.addError('Error parsing _index.html : ' + e)
            return self.report
            
        post_nodes = dom.getElementsByTagNameNS(self.URI, "posts")

        if not post_nodes is None:
            if post_nodes[0].hasAttribute("limit"):
                self.posts_per_page = int(post_nodes[0].getAttribute("limit"))
            else:
                self.posts_per_page = 5
        else:
            self.addError('No tag dyn:posts found')

        now = datetime.datetime.now()
        cur_year = now.year
        posts = Post.objects.filter(creation_date__year=cur_year, published=True, front_page=True).order_by('-creation_date')

        if posts.count() < self.posts_per_page:
            posts = Post.objects.all()[:self.posts_per_page]

        if posts.count() > self.posts_per_page:
            self.nb_pages = self.computeNbPages(posts.count(), self.posts_per_page)

        filename = 'index.html'
        while self.cur_page <= self.nb_pages:
            #print 'Generate ' + filename
            nodes = dom.getElementsByTagName("*")
            nodes[0] = self.parse(src, self.hooks, posts, dom, nodes[0])
            self.writeIfNotTheSame(output + '/' + filename, nodes[0])
            self.cur_page = self.cur_page + 1
            filename = 'index' + str(self.cur_page) + '.html'
            dom = parse(src + '/_index.html')

        filename = output + '/' + filename

        while os.path.exists(filename):
            self.addReport('Removing unused ' + filename)
            os.unlink(filename)
            filename = filename + '.gz'
            if os.path.exists(filename):
                self.addReport('Removing unused ' + filename)
                os.unlink(filename)
            self.cur_page = self.cur_page + 1
            filename = output + '/index' + str(self.cur_page) + '.html'

        if not self.somethingWrote:
            self.addReport('Nothing changed')

        return self.report
